@@ -1,5 +1,6 @@
 """Base class for the elastic analysis of layered snow slabs."""
-# pylint: disable=invalid-name,too-many-arguments,too-many-instance-attributes
+# pylint: disable=invalid-name,too-many-instance-attributes
+# pylint: disable=too-many-arguments,too-many-locals
 
 # Third party imports
 import numpy as np
@@ -104,6 +105,7 @@ class Eigensystem:
         self.kt = False         # Weak-layer shear stiffness
 
         # Initialize slab attributes
+        self.p = 0              # Surface line load (N/mm)
         self.slab = False       # Slab properties dictionary
         self.k = False          # Slab shear correction factor
         self.h = False          # Total slab height (mm)
@@ -123,7 +125,7 @@ class Eigensystem:
         self.sR = False         # Stability shift of real eigenvalues
         self.sysmat = False     # System matrix
 
-    def set_foundation_properties(self, t=30, E=0.25, nu=0.25):
+    def set_foundation_properties(self, t=30, E=0.25, nu=0.25, update=False):
         """
         Set material properties and geometry of foundation (weak layer).
 
@@ -135,6 +137,9 @@ class Eigensystem:
             Weak-layer Young modulus (MPa). Default is 0.25.
         nu : float, optional
             Weak-layer Poisson ratio. Default is 0.25.
+        update : bool, optional
+            If true, recalculate the fundamental system after
+            foundation properties have changed.
         """
         # Geometry
         self.t = t              # Weak-layer thickness (mm)
@@ -145,7 +150,12 @@ class Eigensystem:
             'E': E              # Young's modulus (MPa)
         }
 
-    def set_beam_properties(self, layers, C0=6.0, C1=4.60, nu=0.25):
+        # Recalculate the fundamental system after properties have changed
+        if update:
+            self.calc_fundamental_system()
+
+    def set_beam_properties(self, layers, C0=6.0, C1=4.60,
+                            nu=0.25, update=False):
         """
         Set material and properties geometry of beam (slab).
 
@@ -164,6 +174,9 @@ class Eigensystem:
             Gerling et al. (2017). Default is 4.6.
         nu : float, optional
             Possion's ratio. Default is 0.25
+        update : bool, optional
+            If true, recalculate the fundamental system after
+            foundation properties have changed.
         """
         if isinstance(layers, str):
             # Read layering and Young's modulus from database
@@ -187,6 +200,26 @@ class Eigensystem:
         # Poisson's ratio
         self.slab = np.vstack([layers.T, E, G, nu]).T
 
+        # Recalculate the fundamental system after properties have changed
+        if update:
+            self.calc_fundamental_system()
+
+    def set_surface_load(self, p):
+        """
+        Set surface line load.
+
+        Define a distributed surface load (N/mm) that acts
+        in vertical (gravity) direction on the top surface
+        of the slab.
+
+        Arguments
+        ---------
+        p : float
+            Surface line load (N/mm) that acts in vertical
+            (gravity) direction onm the top surface of the
+            slab.
+        """
+        self.p = p
 
     def calc_foundation_stiffness(self):
         """Compute foundation normal and shear stiffness."""
@@ -267,7 +300,7 @@ class Eigensystem:
         self.sysmat = np.array(E)
 
     def calc_eigensystem(self):
-        """Run eigenvalue analysis of the system matrix."""
+        """Calculate eigenvalues and eigenvectors of the system matrix."""
         # Calculate eigenvalues (ew) and eigenvectors (ev)
         ew, ev = np.linalg.eig(self.sysmat)
         # Classify real and complex eigenvalues
@@ -283,21 +316,28 @@ class Eigensystem:
         self.sR, self.sC = np.zeros(self.ewR.shape), np.zeros(self.ewC.shape)
         self.sR[self.ewR > 0], self.sC[self.ewC > 0] = -1, -1
 
+    def calc_fundamental_system(self):
+        """Calculate the fundamental system of the problem."""
+        self.calc_foundation_stiffness()
+        self.calc_laminate_stiffness_matrix()
+        self.calc_system_matrix()
+        self.calc_eigensystem()
+
     def get_weight_load(self, phi):
         """
-        Calculate line loads.
+        Calculate line loads from slab mass.
 
         Arguments
         ---------
         phi : float
-            Inclination (degrees).
+            Inclination (degrees). Counterclockwise positive.
 
         Returns
         -------
         qn : float
-            Line load (N/mm) in normal direction.
+            Line load (N/mm) at center of gravity in normal direction.
         qt : float
-            Line load (N/mm) in tangential direction.
+            Line load (N/mm) at center of gravity in tangential direction.
         """
         # Convert units
         phi = np.deg2rad(phi)                   # Convert inclination to rad
@@ -306,10 +346,34 @@ class Eigensystem:
         q = sum(rho*self.g*self.slab[:, 1])     # Line load (N/mm)
         # Split into components
         qn = q*np.cos(phi)                      # Normal direction
-        qt = q*np.sin(phi)                      # Tangential direction
+        qt = -q*np.sin(phi)                     # Tangential direction
 
         return qn, qt
 
+    def get_surface_load(self, phi):
+        """
+        Calculate surface line loads.
+
+        Arguments
+        ---------
+        phi : float
+            Inclination (degrees). Counterclockwise positive.
+
+        Returns
+        -------
+        pn : float
+            Surface line load (N/mm) in normal direction.
+        pt : float
+            Surface line load (N/mm) in tangential direction.
+        """
+        # Convert units
+        phi = np.deg2rad(phi)                   # Convert inclination to rad
+        # Split into components
+        pn = self.p*np.cos(phi)                 # Normal direction
+        pt = -self.p*np.sin(phi)                # Tangential direction
+
+        return pn, pt
+
     def get_skier_load(self, m, phi):
         """
         Calculate skier point load.
@@ -319,7 +383,7 @@ class Eigensystem:
         m : float
             Skier weight (kg).
         phi : float
-            Inclination (degrees).
+            Inclination (degrees). Counterclockwise positive.
 
         Returns
         -------
@@ -331,7 +395,7 @@ class Eigensystem:
         phi = np.deg2rad(phi)                   # Convert inclination to rad
         F = 1e-3*np.array(m)*self.g/self.lski   # Total skier load (N)
         Fn = F*np.cos(phi)                      # Normal skier load (N)
-        Ft = F*np.sin(phi)                      # Tangential skier load (N)
+        Ft = -F*np.sin(phi)                     # Tangential skier load (N)
 
         return Fn, Ft
 
@@ -401,32 +465,44 @@ class Eigensystem:
         zp : ndarray
             Particular integral vector (6x1) at position x.
         """
-        # Get weight load
+        # Get weight and surface loads
         qn, qt = self.get_weight_load(phi)
+        pn, pt = self.get_surface_load(phi)
 
-        # Set foundation stiffness variables
+        # Set foundation stiffnesses
         kn = self.kn
         kt = self.kt
 
+        # Unpack laminate stiffnesses
+        A11 = self.A11
+        B11 = self.B11
+        kA55 = self.kA55
+        E0 = self.E0
+
+        # Unpack geometric properties
+        h = self.h
+        t = self.t
+        zs = self.zs
+
         # Assemble particular integral vectors
         if bed:
             zp = np.array([
-                [qt/kt + self.h*qt*(self.h + self.t
-                                    - 2*self.zs)/(4*self.kA55)],
+                [(qt + pt)/kt + h*qt*(h + t - 2*zs)/(4*kA55)
+                    + h*pt*(2*h + t)/(4*kA55)],
                 [0],
-                [qn/kn],
+                [(qn + pn)/kn],
                 [0],
-                [-qt*(self.h + self.t - 2*self.zs)/(2*self.kA55)],
+                [-(qt*(h + t - 2*zs) + pt*(2*h + t))/(2*kA55)],
                 [0]])
         else:
             zp = np.array([
-                [(-3*qt/self.A11 - self.B11*qn*x/self.E0)/6*x**2],
-                [(-2*qt/self.A11 - self.B11*qn*x/self.E0)/2*x],
-                [-self.A11*qn*x**4/(24*self.E0)],
-                [-self.A11*qn*x**3/(6*self.E0)],
-                [self.A11*qn*x**3/(6*self.E0) + (
-                    (self.zs - self.B11/self.A11)*qt - qn*x)/self.kA55],
-                [self.A11*qn*x**2/(2*self.E0) - qn/self.kA55]])
+                [(-3*(qt + pt)/A11 - B11*(qn + pn)*x/E0)/6*x**2],
+                [(-2*(qt + pt)/A11 - B11*(qn + pn)*x/E0)/2*x],
+                [-A11*(qn + pn)*x**4/(24*E0)],
+                [-A11*(qn + pn)*x**3/(6*E0)],
+                [A11*(qn + pn)*x**3/(6*E0)
+                 + ((zs - B11/A11)*qt - h*pt/2 - (qn + pn)*x)/kA55],
+                [(qn + pn)*(A11*x**2/(2*E0) - 1/kA55)]])
 
         return zp
 
