@@ -78,9 +78,18 @@ def outline(grid):
 
 def slab_profile(instance):
     """Create bar chart of slab profile."""
+    # Plot Setup
+    plt.rcdefaults()
+    plt.rc('font', family='serif', size=10)
+    plt.rc('mathtext', fontset='cm')
+
+    # Create figure
+    fig = plt.figure(figsize=(8/3, 4))
+    ax1 = fig.gca()
+
+    # Initialize coordinates
     x = []
     y = []
-
     total_heigth = 0
 
     for line in np.flipud(instance.slab):
@@ -91,24 +100,17 @@ def slab_profile(instance):
         total_heigth = total_heigth + line[1]
         y.append(total_heigth)
 
-    # Font setup
-    plt.rc('font', family='serif', size=8)
-    plt.rc('mathtext', fontset='cm')
-
-    # Create figure
-    fig = plt.figure(figsize=(2, 2))
-    plt.axis()
-    ax1 = fig.gca()
-
     # Set axis labels
-    ax1.set_xlabel('Density (kg/m$^3$)')
-    ax1.set_ylabel('Thickness (mm)')
+    ax1.set_xlabel(r'$\longleftarrow$ Density $\rho$ (kg/m$^3$)')
+    ax1.set_ylabel(r'Height above weak layer (mm) $\longrightarrow$')
 
     ax1.set_xlim(500, 0)
 
     ax1.fill_betweenx(y, 0, x)
     plt.show()
 
+    # Reset plot styles
+    plt.rcdefaults()
 
 # === DEFORMATION CONTOUR PLOT ================================================
 
@@ -130,15 +132,20 @@ def contours(instance, x, z, window=1e12, scale=100):
     scale : int
         Scaling factor for the visualization of displacements.
     """
-    # Calculate y-coordinates (top to bottom)
-    yq = np.linspace(-instance.h/2, instance.h/2, num=21)
+    # Plot Setup
+    plt.rcdefaults()
+    plt.rc('font', family='serif', size=10)
+    plt.rc('mathtext', fontset='cm')
 
-    # Initialize grid coordinates
-    X, Y = np.meshgrid(x/10, yq/10)
+    # Calculate top-to-bottom vertical positions (mm) in beam coordinate system
+    y = np.linspace(-instance.h/2, instance.h/2, num=21)
 
-    # Compute displacements on grid
-    U = np.vstack([instance.u(z, z0=y) for y in yq])
-    W = np.vstack([instance.w(z) for _ in yq])
+    # Compute displacements on grid (cm)
+    U = 1e-1*np.vstack([instance.u(z, z0=y, unit='mm') for y in y])
+    W = 1e-1*np.vstack([instance.w(z, unit='mm') for _ in y])
+
+    # Compute grid coordinates with vertical origin at top surface (cm)
+    X, Y = np.meshgrid(1e-1*x, 1e-1*(y + instance.h/2))
 
     # Plot outline
     plt.plot(outline(X), outline(Y), 'k--', alpha=0.3, linewidth=1)
@@ -158,12 +165,25 @@ def contours(instance, x, z, window=1e12, scale=100):
 
     # Plot setup
     plt.axis('scaled')
-    plt.colorbar(label=r'$u$ ($\mu$m)', shrink=0.5)
-    plt.gca().set_xlabel(r'$x$ (mm)')
-    plt.gca().set_ylabel(r'$z$ (mm)')
+    plt.xlim([xmin, xmax])
+    plt.gca().set_aspect(2)
     plt.gca().invert_yaxis()
     plt.gca().use_sticky_edges = False
-    plt.xlim([xmin, xmax])
+
+    # Plot labels
+    plt.gca().set_xlabel(r'lateral position $x$ (cm) $\longrightarrow$')
+    plt.gca().set_ylabel('depth below surface\n' r'$\longleftarrow $ $d$ (cm)')
+    plt.title(fr'${scale}\!\times\!$ scaled deformations (cm)', size=10)
+
+    # Colorbar
+    cbar = plt.colorbar(
+        shrink=0.5,
+        label=('lateral displacements\n'
+               r'$u$ ($\mu\mathrm{m}$) $\longrightarrow$'))
+    cbar.ax.tick_params(labelsize=8)
+
+    # Reset plot styles
+    plt.rcdefaults()
 
 
 # === BASE PLOT FUNCTION ======================================================
@@ -176,19 +196,17 @@ def plot_data(
         li=False, mi=False, ki=False,
         xlabel=r'Horizontal position $x$ (cm)'):
     """Plot data. Base function."""
-    # Clear figure
-    plt.clf()
+    # Figure setup
+    plt.rcdefaults()
+    plt.rc('font', family='serif', size=10)
+    plt.rc('mathtext', fontset='cm')
 
     # Plot styles
     labelstyle, colors = set_plotstyles()
 
-    # Font setup
-    plt.rc('font', family='serif', size=12)
-    plt.rc('mathtext', fontset='cm')
-
     # Create figure
-    plt.axis()
-    ax1 = plt.gca()
+    fig = plt.figure(figsize=(4, 8/3))
+    ax1 = fig.gca()
 
     # Axis limits
     ax1.autoscale(axis='x', tight=True)
@@ -219,7 +237,7 @@ def plot_data(
         labelpos = int(0.95*len(x[~np.isnan(x)]))
 
     # Fill left y-axis
-    i = -1
+    i = 0
     for x, y, label in ax1data:
         i += 1
         if label == '' or 'FEA' in label:
@@ -270,6 +288,9 @@ def plot_data(
         print('Rendering', filename, '...')
         plt.savefig('plots/' + filename, bbox_inches='tight')
 
+    # Reset plot styles
+    plt.rcdefaults()
+
 
 # === PLOT WRAPPERS ===========================================================
 
@@ -277,11 +298,11 @@ def plot_data(
 def displacements(instance, x, z, **segments):
     """Wrap for dispalcements plot."""
     data = [
-        [x/10, instance.u(z, z0=0), r'$u_0$'],
-        [x/10, -instance.w(z), r'$w$'],
-        [x/10, np.rad2deg(instance.psi(z)), r'$\psi$']
+        [x/10, instance.u(z, z0=0, unit='mm'), r'$u_0\ (\mathrm{mm})$'],
+        [x/10, -instance.w(z, unit='mm'), r'$-w\ (\mathrm{mm})$'],
+        [x/10, instance.psi(z, unit='degrees'), r'$\psi\ (^\circ)$ ']
     ]
-    plot_data(ax1label=r'Displacements (mm)', ax1data=data,
+    plot_data(ax1label=r'Displacements', ax1data=data,
               name='disp', **segments)
 
 
@@ -299,8 +320,8 @@ def section_forces(instance, x, z, **segments):
 def stresses(instance, x, z, **segments):
     """Wrap stress plot."""
     data = [
-        [x/10, 1e3*instance.tau(z), r'$\tau$'],
-        [x/10, 1e3*instance.sig(z), r'$\sigma$']
+        [x/10, instance.tau(z, unit='kPa'), r'$\tau$'],
+        [x/10, instance.sig(z, unit='kPa'), r'$\sigma$']
     ]
     plot_data(ax1label=r'Stress (kPa)', ax1data=data,
               name='stress', **segments)
@@ -351,7 +372,7 @@ def fea_disp(instance, x, z, fea):
         [fea[:, 0]/10,
             np.flipud(np.rad2deg(fea[:, 5])), r'FEA $\psi$'],
         [x/10, instance.u(z, z0=0), r'$u_0$'],
-        [x/10, -instance.w(z), r'$w$'],
+        [x/10, -instance.w(z), r'$-w$'],
         [x/10, np.rad2deg(instance.psi(z)), r'$\psi$']
     ]
     plot_data(
@@ -364,8 +385,8 @@ def fea_stress(instance, xb, zb, fea):
     data = [
         [fea[:, 0]/10, 1e3*np.flipud(fea[:, 2]), r'FEA $\sigma_2$'],
         [fea[:, 0]/10, 1e3*np.flipud(fea[:, 3]), r'FEA $\tau_{12}$'],
-        [xb/10, 1e3*instance.tau(zb), r'$\tau$'],
-        [xb/10, 1e3*instance.sig(zb), r'$\sigma$']
+        [xb/10, instance.tau(zb, unit='kPa'), r'$\tau$'],
+        [xb/10, instance.sig(zb, unit='kPa'), r'$\sigma$']
     ]
     plot_data(ax1label=r'Stress (kPa)', ax1data=data, name='fea_stress',
               labelpos=-50)
